package com.ims.eval.service.custom;

import java.time.LocalDate;
import java.util.*;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import com.ims.eval.dao.*;
import com.ims.eval.entity.*;
import com.ims.eval.service.custom.enums.AIDateTypeEnum;
import lombok.Data;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@RestController
public class AIIntelligentService {
	@Resource
	private EvaluateReportInfoMapper evaluateReportInfoMapper;

	@Resource
	private EvaluateReportMapper evaluateReportMapper;

	@Resource
	private EvaluationDeptMapper evaluationDeptMapper;

	@Resource
	private DeptAssessmentDeclarationMapper deptAssessmentDeclarationMapper;

	@Resource
	private EvaluationPortalMapper evaluationPortalMapper;


	@Value("${AI.scoreType}")
	private String scoreType;


	/**
	 * 考评报告智能生成结果
	 */
	public String intelligentGeneration(String question) {
		StringBuilder sb = new StringBuilder();
		if (question.length() < 5 && (question.contains("你好") || question.contains("您好"))) {
			sb.append("你好！请问有什么我可以帮助你的吗？");
			return sb.toString();
		}
		Map<String, EvaluationDept> evaluateDeptMap = getEvaluateDeptMap();
		for (String deptName : evaluateDeptMap.keySet()) {
			if (question.contains(deptName) || deptName.contains(question)) {
				//部门评级
				String deptId = evaluateDeptMap.get(deptName).getDeptId();
				generationRating(question, sb, deptId);
				return sb.toString();
			}
		}
		//考评报告
		generationReport(question, sb);
		return sb.toString();
	}


	/**
	 * 考评报告智能生成结果
	 */
	public void generationReport(String question, StringBuilder sb) {
		if (question.contains("等级")) {
			getGrade(question, sb);
			return;
		}
		Map<String, EvaluateReportInfo> evaluateReportInfoMap = getEvaluateReportInfoMap();
		Map<String, EvaluateReportInfo> erimap = evaluateReportInfoMap.values().stream().filter(eri -> question.contains(eri.getOrganizationName())).collect(Collectors.toMap(EvaluateReportInfo::getEvaluateReportId, Function.identity()));
		if (erimap.isEmpty()) {
			sb.append("抱歉，这个知识我还在学习中");
			return;
		}
		getReportConcrete(erimap, question, sb);
	}


	/**
	 * 单位等级
	 */
	private void getGrade(String question, StringBuilder sb) {
		Map<String, EvaluationPortal> evaluationPortalMap = getEvaluationPortalMap();
		List<EvaluationPortal> leps = evaluationPortalMap.values().stream().filter(ep -> question.contains(ep.getCompany())).collect(Collectors.toList());
		AIDateInfo ad = getDate(question);
		if (ad.getInfoYear() == null && ad.getInfoQuarter() == null) {
			ad = new AIDateInfo();
			ad.setType(AIDateTypeEnum.Year);
			ad.setInfoYear(LocalDate.now().getYear() + "");
		}
		switch (ad.getType()) {
			case Year:
				getYearGrade(leps, ad, sb);
				break;
			case YearQuarter:
				getYearQuarterGrade(leps, ad, sb);
				break;
			default:
				break;
		}
	}


	private void getYearQuarterGrade(List<EvaluationPortal> leps, AIDateInfo ad, StringBuilder sb) {
		List<EvaluationPortal> lep = leps.stream().filter(ep -> ad.getInfoQuarter().equals(ep.getSeason()) && ad.getInfoYear().equals(ep.getYear())).collect(Collectors.toList());
		if (lep.isEmpty()) {
			sb.append("抱歉，这个知识我还在学习中");
			return;
		}
		sb.append(ad.getInfoYear()).append("年").append(ad.getInfoQuarter()).append("季度").append(lep.get(0).getCompany()).append("的等级是").append(lep.get(0).getGrade());
	}

	private void getYearGrade(List<EvaluationPortal> leps, AIDateInfo ad, StringBuilder sb) {
		List<EvaluationPortal> lep = leps.stream().filter(ep -> ep.getSeason() == null && ad.getInfoYear().equals(ep.getYear())).collect(Collectors.toList());
		if (lep.isEmpty()) {
			sb.append("抱歉，这个知识我还在学习中");
			return;
		}
		sb.append(ad.getInfoYear()).append("年").append(lep.get(0).getCompany()).append("的等级是").append(lep.get(0).getGrade());
	}


	/**
	 * 部门评级智能生成结果
	 */
	public void generationRating(String question, StringBuilder sb, String deptId) {
		Map<String, DeptAssessmentDeclaration> deptAssessmentDeclarationMap = getdeptAssessmentDeclarationMap();
		List<DeptAssessmentDeclaration> ldd = deptAssessmentDeclarationMap.values().stream().filter(dd -> deptId.equals(dd.getDeptId())).collect(Collectors.toList());
		if (ldd.isEmpty()) {
			sb.append("抱歉，这个知识我还在学习中");
			return;
		}
		getRatingConcrete(ldd, question, sb);
	}


	private void getRatingConcrete(List<DeptAssessmentDeclaration> ldd, String question, StringBuilder sb) {
		AIDateInfo ad = getDate(question);
		if (ad.getInfoYear() == null && ad.getInfoMonth() == null) {
			ad = new AIDateInfo();
			ad.setType(AIDateTypeEnum.YearMonth);
			ad.setInfoYear(LocalDate.now().getYear() + "");
			ad.setInfoMonth(((LocalDate.now().getMonth()).getValue() - 2) + "");
		}
		getYearMonthRating(ldd, ad, sb);
	}

	private void getYearMonthRating(List<DeptAssessmentDeclaration> ldd, AIDateInfo ad, StringBuilder sb) {
		int mark = 0;
		int month = (LocalDate.now().getMonth()).getValue() - 1;
		int year = LocalDate.now().getYear();
		if (ldd.isEmpty()) {
			sb.append("抱歉，这个知识我还在学习中");
			return;
		}
		if (ad.getInfoMonth() == null) {
			for (DeptAssessmentDeclaration dd : ldd) {
				if (ad.getInfoYear().equals(dd.getAnnual().toString())) {
					sb.append(dd.getAnnual()).append("年").append(dd.getDeptName()).append(dd.getDeclarationMonth()).append("月").append("考评评级为").append(dd.getRatingGrade()).append("级").append('\n');
				}
			}
			return;
		}
		if (Integer.parseInt(ad.getInfoMonth()) > month && Integer.parseInt(ad.getInfoYear()) >= year) {
			sb.append(ad.getInfoYear()).append("年").append(ad.getInfoMonth()).append("月").append("暂时没有数据，请输入其他年份！");
			return;
		}
		for (DeptAssessmentDeclaration dd : ldd) {
			if (ad.getInfoYear().equals(dd.getAnnual().toString())) {
				if (ad.getInfoMonth().equals(dd.getDeclarationMonth().toString())) {
					sb.append(dd.getAnnual()).append("年").append(dd.getDeptName()).append(ad.getInfoMonth()).append("月").append("考评评级为").append(dd.getRatingGrade()).append("级");
					mark++;
				}
			}
		}
		if (mark == 0) {
			sb.append("抱歉，这个知识我还在学习中");
		}
	}


	/**
	 * 获取具体时间报表
	 */
	private void getReportConcrete(Map<String, EvaluateReportInfo> erimap, String question, StringBuilder sb) {
		List<EvaluateReport> ler = evaluateReportMapper.selectBatchIds(erimap.keySet());
		AIDateInfo ad = getDate(question);
		if (ad.getInfoYear() == null && ad.getInfoQuarter() == null && ad.getInfoMonth() == null) {
			ad = new AIDateInfo();
			ad.setType(AIDateTypeEnum.Year);
			ad.setInfoYear(LocalDate.now().getYear() + "");
		}
		switch (ad.getType()) {
			case Year:
				getYearData(ler, ad, sb, erimap, question);
				break;
			case YearQuarter:
				getYearQuarterData(ler, ad, sb, erimap, question);
				break;
			case YearMonth:
				getYearMonthData(ler, ad, sb, erimap, question);
				break;
			default:
				break;
		}
	}


	/**
	 * 有年和月
	 */
	private void getYearMonthData(List<EvaluateReport> ler, AIDateInfo ad, StringBuilder sb, Map<String, EvaluateReportInfo> erimap, String question) {
		int mark = 0;
		for (EvaluateReport er : ler) {
			if (ad.getInfoYear().equals(er.getYear()) && ad.getInfoMonth().equals(er.getMonth())) {
				sb.append(er.getYear()).append("年").append(er.getMonth()).append("月");
				getInfoScore(sb, er, erimap, question);
				mark++;
				break;
			}
		}
		if (mark == 0) {
			sb.append("抱歉，这个知识我还在学习中");
		}
	}


	/**
	 * 有年和季度
	 */
	private void getYearQuarterData(List<EvaluateReport> ler, AIDateInfo ad, StringBuilder sb, Map<String, EvaluateReportInfo> erimap, String question) {
		int mark = 0;
		for (EvaluateReport er : ler) {
			if (ad.getInfoYear().equals(er.getYear()) && ad.getInfoQuarter().equals(er.getMonth())) {
				sb.append(er.getYear()).append("年").append(er.getMonth()).append("季度");
				getInfoScore(sb, er, erimap, question);
				mark++;
				break;
			}
		}
		if (mark == 0) {
			sb.append("抱歉，这个知识我还在学习中");
		}
	}


	/**
	 * 只有年份
	 */
	private void getYearData(List<EvaluateReport> ler, AIDateInfo ad, StringBuilder sb, Map<String, EvaluateReportInfo> erimap, String question) {
		int mark = 0;
		for (EvaluateReport er : ler) {
			if ("-".equals(er.getMonth()) && ad.getInfoYear().equals(er.getYear())) {
				sb.append(er.getYear()).append("年");
				getInfoScore(sb, er, erimap, question);
				mark++;
				break;
			}
		}
		if (mark == 0) {
			sb.append("抱歉，这个知识我还在学习中");
		}
	}


	/**
	 * 处理接收到的时间
	 */
	public AIDateInfo getDate(String question) {
		AIDateInfo aii = new AIDateInfo();
		// 匹配年份
		Pattern yearPattern = Pattern.compile("(\\d{4}|今年|去年)");
		Matcher yearMatcher = yearPattern.matcher(question);
		if (yearMatcher.find()) {
			String yearStr = yearMatcher.group();
			int year = 0;
			if ("今年".equals(yearStr)) {
				year = LocalDate.now().getYear();
			} else if ("去年".equals(yearStr)) {
				year = LocalDate.now().getYear() - 1;
			} else {
				year = Integer.parseInt(yearStr);
			}
			aii.setType(AIDateTypeEnum.Year);
			aii.setInfoYear(String.valueOf(year));
		}

		// 匹配月份
		Pattern monthPattern = Pattern.compile("(\\d{1,2}月)");
		Matcher monthMatcher = monthPattern.matcher(question);
		if (monthMatcher.find()) {
			String monthStr = monthMatcher.group().replace("月", "");
			if (aii.getInfoYear() == null) {
				aii.setInfoYear(LocalDate.now().getYear() + "");
			}
			aii.setType(AIDateTypeEnum.YearMonth);
			aii.setInfoMonth(monthStr);
		}

		// 匹配季度
		Pattern quarterPattern = Pattern.compile("(第)?([一-四]|\\d{1,2})季度");
		Matcher quarterMatcher = quarterPattern.matcher(question);
		if (quarterMatcher.find()) {
			String quarterStr = quarterMatcher.group(2);
			int quarterNum = convertToQuarterNumber(quarterStr);
			if (quarterNum > 0) {
				if (aii.getInfoYear() == null) {
					aii.setInfoQuarter(LocalDate.now().getYear() + "");
				}
				aii.setType(AIDateTypeEnum.YearQuarter);
				aii.setInfoQuarter(quarterNum + "");
			}
		}
		return aii;
	}


	private void getInfoScore(StringBuilder sb, EvaluateReport er, Map<String, EvaluateReportInfo> erimap, String question) {
		int mark = 0;
		EvaluateReportInfo e = erimap.get(er.getId());
		sb.append(e.getOrganizationName()).append("的");
		double s3 = 0.0;
		String[] sType = scoreType.split(",");
		for (String s2 : sType) {
			if (question.contains(s2)) {
				switch (s2) {
					case "最终得分":
						s3 = e.getScore();
						sb.append("最终得分").append(s3).append("分");
						break;
					case "综合得分":
						s3 = e.getZhScore();
						sb.append("综合得分").append(s3).append("分");
						break;
					case "生产经营得分":
						s3 = e.getScjyScore();
						sb.append("生产经营得分").append(s3).append("分");
						break;
					case "前期得分":
						s3 = e.getQqScore();
						sb.append("前期得分").append(s3).append("分");
						break;
					case "基建得分":
						s3 = e.getJjScore();
						sb.append("基建得分").append(s3).append("分");
						break;
					case "加分项":
						s3 = e.getAddScore();
						sb.append("加分项").append(s3).append("分");
						break;
					case "减分项":
						s3 = e.getSubScore();
						sb.append("减分项").append(s3).append("分");
						break;
					case "加减分合计":
						s3 = e.getAddSubScore();
						sb.append("加减分合计").append(s3).append("分");
						break;
					case "经营业绩系数":
						s3 = e.getJyyjXs();
						sb.append("经营业绩系数").append(s3).append("分");
						break;
					case "关键业绩系数":
						s3 = e.getGjyjXs();
						sb.append("关键业绩系数").append(s3).append("分");
						break;
					case "党建考核乘数":
						s3 = e.getDjkhCs();
						sb.append("党建考核乘数").append(s3).append("分");
						break;
					case "折算分数":
						s3 = e.getDiscountScore();
						sb.append("折算分数").append(s3).append("分");
						break;
				}
				mark++;
			}
		}
		if (mark == 0) {
			sb.append("最终得分").append(e.getScore()).append("分").append(",").append("综合得分").append(e.getZhScore()).append("分")
				.append(",").append("生产经营得分").append(e.getScjyScore()).append("分").append(",")
				.append("前期得分").append(e.getQqScore()).append("分").append(",").append("基建得分").append(e.getJjScore()).append("分");
		}
	}


	private int convertToQuarterNumber(String quarterStr) {
		switch (quarterStr) {
			case "一":
			case "1":
				return 1;
			case "二":
			case "2":
				return 2;
			case "三":
			case "3":
				return 3;
			case "四":
			case "4":
				return 4;
			default:
				// 如果不是有效的季度数字，则返回0或抛出异常
				return 0;
		}
	}


	/**
	 * 考评详情map
	 */
	public Map<String, EvaluateReportInfo> getEvaluateReportInfoMap() {
		List<EvaluateReportInfo> evaluateReportInfos = evaluateReportInfoMapper.selectList(null);
		return evaluateReportInfos.stream().collect(Collectors.toMap(EvaluateReportInfo::getId, Function.identity()));
	}

	/**
	 * 单位等级map
	 */
	public Map<String, EvaluationPortal> getEvaluationPortalMap() {
		List<EvaluationPortal> evaluateReportInfos = evaluationPortalMapper.selectList(null);
		return evaluateReportInfos.stream().collect(Collectors.toMap(EvaluationPortal::getId, Function.identity()));
	}


	/**
	 * 部门map
	 */
	public Map<String, EvaluationDept> getEvaluateDeptMap() {
		List<EvaluationDept> evaluationDepts = evaluationDeptMapper.selectList(null);
		return evaluationDepts.stream().collect(Collectors.toMap(EvaluationDept::getDeptName, Function.identity()));
	}


	/**
	 * 部门评级map
	 */
	public Map<String, DeptAssessmentDeclaration> getdeptAssessmentDeclarationMap() {
		List<DeptAssessmentDeclaration> evaluationDeptRatings = deptAssessmentDeclarationMapper.selectList(null);
		return evaluationDeptRatings.stream().collect(Collectors.toMap(DeptAssessmentDeclaration::getId, Function.identity()));
	}


	@Data
	public static class AIDateInfo {
		private String infoYear;
		private String infoMonth;
		private String infoQuarter;
		private AIDateTypeEnum type;
	}


}
