package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.EvaluateRuleMapper;
import com.ims.eval.entity.BinSection;
import com.ims.eval.entity.BinStage;
import com.ims.eval.entity.EvaluateRule;
import com.ims.eval.entity.dto.response.EvaluateRuleInfoResDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IBinSectionService;
import com.ims.eval.service.IBinStageService;
import com.ims.eval.service.IEvaluateRuleService;
import com.ims.eval.util.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 考评规则表 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-02-27
 */
@Service
public class EvaluateRuleServiceImpl extends ServiceImpl<EvaluateRuleMapper, EvaluateRule> implements IEvaluateRuleService {

	@Autowired
	private IBinStageService binStageService;

	@Autowired
	private IBinSectionService binSectionService;

	@Override
	public IPage<EvaluateRule> list(Integer pageNum, Integer pageSize, String id, String des, String binSection, String binStage) {

		QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}

		//构造分页构造器
		Page<EvaluateRule> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(EvaluateRule::getId, id);
		}

		if (StringUtils.isNotEmpty(des)) {
			qw.lambda().like(EvaluateRule::getDes, des);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().like(EvaluateRule::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			qw.lambda().eq(EvaluateRule::getBinStage, binStage);
		}

		qw.lambda().orderByDesc(EvaluateRule::getCreateTime);

		IPage<EvaluateRule> list = baseMapper.selectPage(page, qw);
		list.getRecords().stream().forEach(l -> {
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" : CacheContext.ddNameMap.get(l.getBinSection()));
			l.setBinStageName(null == CacheContext.ddNameMap.get(l.getBinStage()) ? "" : CacheContext.ddNameMap.get(l.getBinStage()));
		});
		return list;
	}

	@Override
	public IPage<EvaluateRule> listPage(Integer pageNum, Integer pageSize, String id, String des, String binSection, String binStage, String checkCycle, String year) {
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		//构造分页构造器
		Page<EvaluateRule> page = new Page<>(pageNum, pageSize);
		IPage<EvaluateRule> list = baseMapper.selectListPage(page, id, des, binSection, binStage, checkCycle, year);
		return list;
	}

	@Override
	public List<EvaluateRule> listAll(String id, String des, String binSection, String binStage, String checkCycle, String year) {
		QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(EvaluateRule::getId, id);
		}

		if (StringUtils.isNotEmpty(des)) {
			qw.lambda().like(EvaluateRule::getDes, des);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			List<String> binSections = Arrays.asList(binSection.split(","));

			qw.lambda().in(EvaluateRule::getBinSection, binSections);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			List<String> binStages = Arrays.asList(binStage.split(","));

			qw.lambda().in(EvaluateRule::getBinStage, binStages);
		}


		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(EvaluateRule::getCheckCycle, checkCycle);
		}

		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(EvaluateRule::getYear, year);
		}

		qw.lambda().orderByDesc(EvaluateRule::getCreateTime);

		List<EvaluateRule> list = baseMapper.selectList(qw);

		list.stream().forEach(l -> {
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" : CacheContext.ddNameMap.get(l.getBinSection()));
			l.setBinStageName(null == CacheContext.ddNameMap.get(l.getBinStage()) ? "" : CacheContext.ddNameMap.get(l.getBinStage()));
		});

		return list;
	}

	@Override
	public List<EvaluateRule> listAll(List<String> ids) {
		QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();

		if (null == ids || ids.size() <= 0) {
			throw new CustomException("获取规则参数空");
		}
		qw.lambda().in(EvaluateRule::getId, ids);
		List<EvaluateRule> list = baseMapper.selectList(qw);
		return list;
	}

	@Override
	public List<EvaluateRuleInfoResDTO> getListInIdsInfos(List<String> ids, String binSection, String binStage) {
		List<EvaluateRuleInfoResDTO> list = baseMapper.selectListInIdsInfos(ids, binSection, binStage);
		return list;
	}

	@Override
	public List<EvaluateRule> getEvaluateRuleByYear(String binSection, String binStage, String checkCycle, String year) {


		QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(binSection)) {
			List<String> binSections = Arrays.asList(binSection.split(","));

			qw.lambda().in(EvaluateRule::getBinSection, binSections);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			List<String> binStages = Arrays.asList(binStage.split(","));
			qw.lambda().in(EvaluateRule::getBinStage, binStages);
		}

		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(EvaluateRule::getCheckCycle, checkCycle);
		}

		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(EvaluateRule::getYear, year);
		}

		qw.lambda().eq(EvaluateRule::getEnable, true);
		qw.lambda().orderByDesc(EvaluateRule::getCreateTime);

		List<EvaluateRule> list = baseMapper.selectList(qw);
		return list;

	}

	@Override
	public boolean saveOrUpdate(EvaluateRule entity) {

		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();
			qw.lambda().eq(EvaluateRule::getBinSection, entity.getBinSection());
			qw.lambda().eq(EvaluateRule::getBinStage, entity.getBinStage());
			qw.lambda().eq(EvaluateRule::getEnable, true);
			qw.lambda().eq(EvaluateRule::getYear, entity.getYear());
			qw.lambda().eq(EvaluateRule::getCheckCycle, entity.getCheckCycle());
			List<EvaluateRule> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("当前模块规则已存在");
			}

			entity.setCreateTime(new Date());//创建时间
			entity.setEnable(true);//默认启用
		} else {
			entity.setUpdateTime(new Date());
		}
		return super.saveOrUpdate(entity);

	}

	@Override
	public List<EvaluateRule> getEvaluateRuleList(String binSection, String binStage, String checkCycle, String year) {
		List<EvaluateRule> list = baseMapper.selectEvaluateRuleList(binSection, binStage, checkCycle, year);
		return list;
	}

	@Override
	public EvaluateRule getEvaluateRuleListByYear(String year, String checkCycle, String manageCategory) {
		if ("火电".equals(manageCategory)) {
			manageCategory = "HD";
		} else if ("水电".equals(manageCategory)) {
			manageCategory = "SD";
		} else if ("新能源".equals(manageCategory)) {
			manageCategory = "XNYFG";
		} else if ("风电".equals(manageCategory)) {
			manageCategory = "FD";
		} else if ("光伏".equals(manageCategory)) {
			manageCategory = "GD";
		} else if ("煤电".equals(manageCategory)) {
			manageCategory = "MDYTH";
		} else if ("海外".equals(manageCategory)) {
			manageCategory = "GJYW";
		} else if ("综合".equals(manageCategory)) {
			manageCategory = "ZHZC";
		}
		return baseMapper.getEvaluateRuleListByYear(year, checkCycle, manageCategory);
	}

	@Override
	public void exportExcel(String binSection, String checkCycle, String year, HttpServletResponse response) throws IOException {
		// 设置响应头信息，以附件形式下载
		response.setHeader("Content-Disposition", "attachment; filename=evaluate_rule.xlsx");
		response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");

		// 创建新的Excel工作簿
		Workbook workbook = new XSSFWorkbook();

		// 创建新的工作表
		Sheet sheet = workbook.createSheet("Sheet1");

		// 创建表头
		Row headerRow = sheet.createRow(0);
		List<String> headers = Arrays.asList("规则名称", "业务阶段", "业务属性", "考评周期", "年份", "描述");
		for (int i = 0; i < headers.size(); i++) {
			Cell cell = headerRow.createCell(i);
			cell.setCellValue(headers.get(i));
		}

		//行数据
		List<EvaluateRule> evaluateRuleList = baseMapper.selectEvaluateRuleList(binSection, "", checkCycle, year);

		// 填充数据到工作表
		int rowIndex = 1; // 从第二行开始填充数据（第一行是表头）
		for (EvaluateRule evaluateRule : evaluateRuleList) {
			Row row = sheet.createRow(rowIndex++);

			// 设置数据
			row.createCell(0).setCellValue(evaluateRule.getRuleName());
			row.createCell(1).setCellValue(evaluateRule.getBinStageName());
			row.createCell(2).setCellValue(evaluateRule.getBinSectionName());
			if ("NDKP".equals(evaluateRule.getCheckCycle())) {
				checkCycle = "年度考评";
			} else {
				checkCycle = "月/季度考评";
			}
			row.createCell(3).setCellValue(checkCycle);
			row.createCell(4).setCellValue(evaluateRule.getYear());
			row.createCell(5).setCellValue(evaluateRule.getDes());
		}

		// 自动调整列宽
		for (int i = 0; i < headers.size(); i++) {
			sheet.autoSizeColumn(i);
		}

		// 将工作簿写入到输出流
		try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
			workbook.write(outputStream);
			response.getOutputStream().write(outputStream.toByteArray());
			response.getOutputStream().flush();
		}

		// 关闭工作簿
		workbook.close();
	}

	@Override
	public void importTemplate(HttpServletResponse response) throws IOException {
		// 设置响应头信息，以附件形式下载
		response.setHeader("Content-Disposition", "attachment; filename=evaluate_rule.xlsx");
		response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");

		// 创建新的Excel工作簿
		Workbook workbook = new XSSFWorkbook();

		// 创建新的工作表
		Sheet sheet = workbook.createSheet("Sheet1");

		// 创建表头
		Row headerRow = sheet.createRow(0);
		List<String> headers = Arrays.asList("规则名称", "业务阶段", "业务属性", "考评周期", "年份", "描述");
		for (int i = 0; i < headers.size(); i++) {
			Cell cell = headerRow.createCell(i);
			cell.setCellValue(headers.get(i));
		}

		// 将工作簿写入到输出流
		try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
			workbook.write(outputStream);
			response.getOutputStream().write(outputStream.toByteArray());
			response.getOutputStream().flush();
		}

		// 关闭工作簿
		workbook.close();
	}

	@Override
	public R importExcel(MultipartFile file) {
		if (!file.isEmpty()) {
			try {
				//获取原始的文件名
				String originalFilename = file.getOriginalFilename();
				//获取文件类型
				String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
				//默认从第一行开始读取
				int startRows = 1;
				//获取输入流
				InputStream is = file.getInputStream();
				List<EvaluateRule> evaluateRuleList = new ArrayList<>();
				//Excel导入导出的单元类
				List<String[]> strings = ExcelUtil.readData(fileType, startRows, true, is);
				//遍历Excel表每一行的数据
				for (String[] str : strings) {
					EvaluateRule evaluateRule = new EvaluateRule();
					String checkCycle = "";
					if ("年度考评".equals(str[3])) {
						checkCycle = "NDKP";
					} else if ("月/季度考评".equals(str[3])) {
						checkCycle = "JDKP";
					} else {
						checkCycle = "JDKP";
					}
					BinStage binStage = binStageService.getBinStageByStageName(str[1]);
					BinSection binSection = binSectionService.getBinSectionBySectionName(str[2]);
					if (null != binStage && null != binSection){
						EvaluateRule obj = this.checkForExistence(str[0], checkCycle, binStage.getId(), binSection.getId(), str[4]);
						if (null != obj) {
							evaluateRule.setId(obj.getId());
							evaluateRule.setRuleName(str[1]);
							evaluateRule.setDes(str[6]);
							evaluateRuleList.add(evaluateRule);
						} else {
							evaluateRule.setRuleName(str[0]);
							evaluateRule.setBinStage(binStage.getId());
							evaluateRule.setBinSection(binSection.getId());
							evaluateRule.setEnable(true);
							evaluateRule.setCheckCycle(checkCycle);
							evaluateRule.setYear(str[4]);
							evaluateRule.setDes(str[5]);
							evaluateRule.setCreateTime(new Date());
							evaluateRuleList.add(evaluateRule);
						}
					}
				}
				boolean b = super.saveOrUpdateBatch(evaluateRuleList);
				if (b) {
					return R.ok().data(b);
				} else {
					return R.error("导入失败！");
				}
			} catch (Exception e) {
				return R.customError(e.getMessage()).data("失败！");
			}
		}
		return R.error("上传文件为空！");
	}

	@Override
	public EvaluateRule getEvaluateRuleByRuleName(String evaluateRuleName, String year) {
		QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(evaluateRuleName)) {
			qw.lambda().eq(EvaluateRule::getRuleName, evaluateRuleName);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(EvaluateRule::getYear, year);
		}
		return baseMapper.selectOne(qw);
	}

	@Override
	public EvaluateRule checkForExistence(String ruleName, String checkCycle, String binStage, String binSection, String year) {
		QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(ruleName)) {
			qw.lambda().eq(EvaluateRule::getRuleName, ruleName);
		}
		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(EvaluateRule::getCheckCycle, checkCycle);
		}
		if (StringUtils.isNotEmpty(binStage)) {
			qw.lambda().eq(EvaluateRule::getBinStage, binStage);
		}
		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(EvaluateRule::getBinSection, binSection);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(EvaluateRule::getYear, year);
		}
		return baseMapper.selectOne(qw);
	}
}
